@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2015-04-07'; # DATE
-our $VERSION = '1.05'; # VERSION
+our $DATE = '2015-04-09'; # DATE
+our $VERSION = '1.06'; # VERSION
 
 use 5.010001;
 use strict;
@@ -77,15 +77,18 @@ has cleanser => (
 );
 
 # role: requires 'hook_after_get_meta'
-# role: requires 'hook_before_run'
-# role: optional 'hook_before_read_config_file'
-# role: requires 'hook_after_parse_argv'
-# role: optional 'hook_after_read_config_file'
-# role: requires 'hook_format_result'
 # role: requires 'hook_format_row'
+# role: requires 'default_prompt_template'
+
+# role: requires 'hook_before_run'
+# role: requires 'hook_before_read_config_file'
+# role: requires 'hook_after_read_config_file'
+# role: requires 'hook_after_parse_argv'
+# role: requires 'hook_before_action'
+# role: requires 'hook_after_action'
+# role: requires 'hook_format_result'
 # role: requires 'hook_display_result'
 # role: requires 'hook_after_run'
-# role: requires 'default_prompt_template'
 
 # we put common stuffs here, but PC::Classic's final version will differ from
 # PC::Lite's in several aspects: translation, supported output formats,
@@ -339,10 +342,16 @@ sub __default_env_name {
     "${prog}_OPT";
 }
 
+sub hook_before_run {}
+
 sub hook_before_read_config_file {}
 
 sub hook_after_read_config_file {}
 
+sub hook_before_action {}
+
+sub hook_after_action {}
+
 sub get_meta {
     my ($self, $r, $url) = @_;
 
@@ -1136,7 +1145,9 @@ sub run {
 
         $log->tracef("[pericmd] Running hook_after_parse_argv ...");
         $self->hook_after_parse_argv($r);
+
         $self->parse_cmdline_src($r);
+
         my $missing = $parse_res->[3]{"func.missing_args"};
         die [400, "Missing required argument(s): ".join(", ", @$missing)]
             if $missing && @$missing;
@@ -1147,11 +1158,17 @@ sub run {
             $r->{args}{-cmdline_r} = $r;
         }
 
-        my $meth = "run_$r->{action}";
+        $log->tracef("[pericmd] Running hook_before_action ...");
+        $self->hook_before_action($r);
+
+        my $meth = "action_$r->{action}";
         die [500, "Unknown action $r->{action}"] unless $self->can($meth);
         $log->tracef("[pericmd] Running %s() ...", $meth);
         $r->{res} = $self->$meth($r);
         #$log->tracef("[pericmd] res=%s", $r->{res}); #1
+
+        $log->tracef("[pericmd] Running hook_after_action ...");
+        $self->hook_after_action($r);
     };
     my $err = $@;
     if ($err || !$r->{res}) {
@@ -1231,7 +1248,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{Classic,::Lite}
 
 =head1 VERSION
 
-This document describes version 1.05 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-04-07.
+This document describes version 1.06 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-04-09.
 
 =head1 DESCRIPTION
 
@@ -1316,15 +1333,20 @@ Function arguments that are still missing can be filled from STDIN or files, if
 the metadata specifies C<cmdline_src> property (see L<Rinci::function> for more
 details).
 
-=item * Delegate to C<run_$action> method
+=item * Delegate to C<action_$action> method
+
+Before running the C<action_$action> method, C<hook_before_action> is called
+e.g. to allow changing/fixing action, last chance to check arguments, etc.
 
 After we get the action from the previous step, we delegate to separate
-C<run_$action> method (so there is C<run_version>, C<run_help>, and so on; and
-also C<run_call>). These methods also receive C<$r> as their argument and must
-return an enveloped result (see L<Rinci::function> for more details).
+C<action_$action> method (so there is C<action_version>, C<action_help>, and so
+on; and also C<action_call>). These methods also receive C<$r> as their argument
+and must return an enveloped result (see L<Rinci::function> for more details).
 
 Result is put in C<< $r->{res} >>.
 
+C<hook_after_action> is then called e.g. to preformat result.
+
 =item * Run hook_format_result
 
 Hook must set C<< $r->{fres} >> (formatted result).
@@ -1332,8 +1354,6 @@ Hook must set C<< $r->{fres} >> (formatted result).
 If result has C<cmdline.skip_format> result metadata property, then this step is
 skipped and C<< $r->{fres} >> is simply taken from C<< $r->{res}[2] >>.
 
-A C<run_$action> method can set
-
 =item * Run hook_display_result
 
 This hook is used by XXX.
@@ -1730,7 +1750,7 @@ per-subcommand basis.
 In addition to C<-cmdline>, C<-cmdline_r> will also be passed, containing the
 C<$r> per-request stash/hash (see L</"REQUEST KEYS">).
 
-Passing the cmdline object can be useful, e.g. to call run_help(), to get the
+Passing the cmdline object can be useful, e.g. to call action_help(), to get the
 settings of the Perinci::CmdLine, etc.
 
 =head2 program_name => str
@@ -1923,21 +1943,59 @@ Only called when C<read_config> attribute is true.
 
 Only called when C<read_config> attribute is true.
 
+=head2 $cmd->hook_after_get_meta($r)
+
+Called after the C<get_meta> method gets function metadata, which normally
+happens during parsing argument, because parsing function arguments require the
+metadata (list of arguments, etc).
+
+PC:Lite as well as PC:Classic use this hook to insert a common option
+C<--dry-run> if function metadata expresses that function supports dry-run mode.
+
+PC:Lite also checks the C<deps> property here. PC:Classic doesn't do this
+because it uses function wrapper (L<Perinci::Sub::Wrapper>) which does this.
+
 =head2 $cmd->hook_after_parse_argv($r)
 
 Called after C<run()> calls C<parse_argv()> and before it checks the result.
 C<$r->{parse_argv_res}> will contain the result of C<parse_argv()>. The hook
 gets a chance to, e.g. fill missing arguments from other source.
 
+Note that for sources specified in the C<cmdline_src> property, this base class
+will do the filling in after running this hook, so no need to do that here.
+
+PC:Lite uses this hook to give default values to function arguments C<<
+$r->{args} >> from the Rinci metadata. PC:Classic doesn't do this because it
+uses function wrapper (L<Perinci::Sub::Wrapper>) which will do this as well as
+some other stuffs (validate function arguments, etc).
+
+=head2 $cmd->hook_before_action($r)
+
+Called before calling the C<action_ACTION> method. Some ideas to do in this
+hook: modifying action to run (C<< $r->{action} >>), last check of arguments
+(C<< $r->{args} >>) before passing them to function.
+
+PC:Lite uses this hook to validate function arguments. PC:Classic does not do
+this because it uses function wrapper which already does this.
+
+=head2 $cmd->hook_after_action($r)
+
+Called after calling C<action_ACTION> method. Some ideas to do in this hook:
+preformatting result (C<< $r->{res} >>).
+
 =head2 $cmd->hook_format_result($r)
 
 The hook is supposed to format result in C<$res->{res}> (an array).
 
+All direct subclasses of PC:Base do the formatting here.
+
 =head2 $cmd->hook_display_result($r)
 
 The hook is supposed to display the formatted result (stored in C<$r->{fres}>)
 to STDOUT. But in the case of streaming output, this hook can also set it up.
 
+All direct subclasses of PC:Base do the formatting here.
+
 =head2 $cmd->hook_after_run($r)
 
 Called at the end of C<run()>, right before it exits (if C<exit> attribute is
