@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Lite;
 
-our $DATE = '2015-04-07'; # DATE
-our $VERSION = '1.05'; # VERSION
+our $DATE = '2015-04-09'; # DATE
+our $VERSION = '1.06'; # VERSION
 
 use 5.010001;
 # use strict; # already enabled by Mo
@@ -140,8 +140,6 @@ sub _unsetup_progress_output {
     $setup_progress = 0;
 }
 
-sub hook_before_run {}
-
 sub hook_after_parse_argv {
     my ($self, $r) = @_;
 
@@ -171,6 +169,53 @@ sub hook_after_parse_argv {
     }
 }
 
+sub hook_before_action {
+    my ($self, $r) = @_;
+
+    # validate arguments using schema from metadata
+  VALIDATE_ARGS:
+    {
+        # unless we're feeding the arguments to function, don't bother
+        # validating arguments
+        last unless $r->{action} eq 'call';
+
+        my $meta = $r->{meta};
+
+        # function is probably already wrapped
+        last if $meta->{'x.perinci.sub.wrapper.logs'} &&
+            (grep { $_->{validate_args} }
+             @{ $meta->{'x.perinci.sub.wrapper.logs'} });
+
+        require Data::Sah;
+
+        # to be cheap, we simply use "$ref" as key as cache key. to be proper,
+        # it should be hash of serialized content.
+        my %validators; # key = "$schema"
+
+        for my $arg (sort keys %{ $meta->{args} // {} }) {
+            next unless exists($r->{args}{$arg});
+
+            # we don't support validation of input stream because this must be
+            # done after each 'get item' (but periswrap does)
+            next if $meta->{args}{$arg}{stream};
+
+            my $schema = $meta->{args}{$arg}{schema};
+            next unless $schema;
+            unless ($validators{"$schema"}) {
+                my $v = Data::Sah::gen_validator($schema, {
+                    return_type => 'str',
+                    schema_is_normalized => 1,
+                });
+                $validators{"$schema"} = $v;
+            }
+            my $res = $validators{"$schema"}->($r->{args}{$arg});
+            if ($res) {
+                die [400, "Argument '$arg' fails validation: $res"];
+            }
+        }
+    }
+}
+
 sub __json {
     state $json = do {
         require JSON;
@@ -384,7 +429,7 @@ sub hook_after_get_meta {
     }
 }
 
-sub run_subcommands {
+sub action_subcommands {
     my ($self, $r) = @_;
 
     if (!$self->subcommands) {
@@ -403,7 +448,7 @@ sub run_subcommands {
       )];
 }
 
-sub run_version {
+sub action_version {
     my ($self, $r) = @_;
 
     my $meta = $r->{meta} = $self->get_meta($r, $self->url);
@@ -421,7 +466,7 @@ sub run_version {
       )];
 }
 
-sub run_help {
+sub action_help {
     require Perinci::CmdLine::Help;
 
     my ($self, $r) = @_;
@@ -454,7 +499,7 @@ sub run_help {
     $res;
 }
 
-sub run_call {
+sub action_call {
     my ($self, $r) = @_;
 
     my %extra;
@@ -498,7 +543,7 @@ Perinci::CmdLine::Lite - A lightweight Rinci/Riap-based command-line application
 
 =head1 VERSION
 
-This document describes version 1.05 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2015-04-07.
+This document describes version 1.06 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2015-04-09.
 
 =head1 SYNOPSIS
 
@@ -622,7 +667,7 @@ Perinci::CmdLine::Lite is the default backend implementation. You normally
 should use L<Perinci::CmdLine::Any> instead to be able to switch backend on the
 fly.
 
-=for Pod::Coverage ^(BUILD|get_meta|hook_.+|run_.+)$
+=for Pod::Coverage ^(BUILD|get_meta|hook_.+|action_.+)$
 
 =head1 REQUEST KEYS
 
